apply_labels = function(data.frame, labels.list = labels)
{  
  for(col in names(labels.list)[names(labels.list) != "columns"])
  {
    ## Only apply the transformation if the column exists, otherwise skip the column
    if(col %in% names(data.frame))
    {
      ## If the corresponding column in the data frame is not a factor, then make it one
      if(!is.factor(data.frame[,col]))
      {
        data.frame[,col] <- as.factor(data.frame[,col])
      }
      
      ## Get levels and respective labels as from the label list
      levels = names(labels.list[[col]])
      labels = as.character(labels.list[[col]])
      
      ## Pad missing labels with values as they appear in data
      if(!all(levels(data.frame[,col]) %in% names(labels.list[[col]])))
      {
        ## Levels of data.frame, where levels that appear in labels appear first and ordered as in the list
        levels = unique(c(names(labels.list[[col]]), levels(data.frame[,col])))
        #levels = names(factor(data.frame[,col], levels=unique(c(labels.list[[col]], levels(data.frame[,col])))))
        
        ## Concatenate labels with Levels of data.frame that are not in the list of labels
        labels = c(as.character(labels.list[[col]]), levels(factor(data.frame[,col][!data.frame[,col] %in% names(labels.list[[col]])])))
      }

      ## Reorder levels of factors in data.frame, and give them labels
      data.frame[,col] <- factor(data.frame[,col], levels=levels, labels=labels)
      
      ## Only keep factors actually used in the table
      data.frame[,col] <- factor(data.frame[,col])
    }
  }
  
  return(data.frame)
}

label = function(col, columns="columns", labels.list = labels)
{
  if(col %in% names(labels.list[[columns]]))
    return(labels.list[[columns]][col])
  else
    return(col)
}

labels = list(
	scheduler = c("crown_integrated_packed__noidle" = "Cons.Integ.(a=3.00,z=0.65,e=0.00,k=52.64)", "crown_integrated_packed__highidle" = "Cons.Integ.(a=3.00,z=0.65,e=0.50,k=52.64)"),
	evaluation = c("static_dynamic_busy_active__highidle" = "Stat./dyn.a=3,ep=0.649,k=52.6392,eta=0.5(act.)"),
	platform = c("intel_i5_4690k_haswell_p2" = "2", "intel_i5_4690k_haswell_p4" = "4", "intel_i5_4690k_haswell_p8" = "8"),
	application = c("audiobeam" = "AUDIOBEAM", "beamformer" = "BEAMFORMER", "BubbleSort" = "BUBBLESORT", "fir" = "FIR", "filterbank" = "FILTERBANK", "beamformer" = "BEAMFORMER", "vocoder" = "VOCODER", "tconvolve" = "TCONVOLVE"),
	columns = c("scheduler" = "scheduler", "evaluation" = "evaluation", "taskgraph" = "taskgraph", "platform" = "platform", "schedule_try" = "schedule try", "evaluation_try" = "evaluation try", "schedulable" = "schedulable", "evaluable" = "evaluable", "n" = "n", "p" = "p", "work_stddev" = "work stddev", "max_width_mean" = "max width mean", "max_width_stddev" = "max width stddev", "application" = "application", "optimal" = "optimal", "feasible" = "feasible", "time_allocation" = "time allocation", "time_mapping" = "time mapping", "time_frequency" = "time frequency", "time_global" = "time global", "quality_mapping" = "quality mapping", "complexity" = "complexity", "switching" = "switching", "switching_energy" = "switching energy", "valid_schedule" = "valid schedule", "time" = "time", "power" = "power")
)
