/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <iostream>
#include <cstdlib>
#include <fstream>
#include <string>
#include <boost/regex.hpp>
#include <iomanip>

#include <crown/CrownUtil.hpp>

#include <pelib/AmplInput.hpp>
#include <pelib/AmplOutput.hpp>

#include <pelib/AmplOutputScalar.hpp>
#include <pelib/AmplOutputVector.hpp>
#include <pelib/AmplOutputSet.hpp>
#include <pelib/AmplOutputMatrix.hpp>

#include <pelib/AmplInputScalar.hpp>
#include <pelib/AmplInputVector.hpp>
#include <pelib/AmplInputSet.hpp>
#include <pelib/AmplInputMatrix.hpp>

using namespace pelib;
using namespace pelib::crown;
using namespace std;

const int b = 2;
typedef map<int, int> TaskWidthVector;
typedef map<int, int> TaskScheduleVector;
typedef map<int, TaskScheduleVector> ProcScheduleMatrix;

int
group_size(int group, int p, int b)
{
	return (int)(p / pow(b, floor((log(group) / log(b)))));
}

Algebra
parse(AlgebraParser &parser, std::istream &input)
{
	Algebra rec;
	
	try {
		rec = parser.parse(input);
	} catch(ParseException &e)
	{
		std::cerr << e.what() << std::endl;
	}

	return rec;
}

int
main(int argc, char **argv)
{
	AmplOutput input(AmplOutput::floatHandlers());
	AmplOutput output(AmplOutput::floatHandlers());
	Algebra rec;

	// Read from file
	/*
	std::ifstream myfile;
	myfile.open (argv[1], std::ios::in);
	rec = parse(input, myfile);
	myfile.close(); */
	rec = parse(input, cin);

	cout << setprecision(6)
	<< setiosflags(ios::fixed)
	<< setiosflags(ios::showpoint);

	rec = CrownUtil::crownToSchedule(rec);

	// Outputs all useful information for schedule evaluation
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Set<float> >("F"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	{
		const Scalar<float> *M = rec.find<Scalar<float> >("M");
		if(M != NULL)
		{
			output.dump(cout, M);
		}
	}
	{
		const Scalar<float> *M = rec.find<Scalar<float> >("M");
		if(M != NULL)
		{
			output.dump(cout, M);
		}
	}
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Scalar<float> >("n"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Scalar<float> >("p"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Vector<int, float> >("Tau"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Vector<int, float> >("wi"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Vector<int, float> >("frequency"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	output.dump(cout, rec.find<Matrix<int, int, float> >("e"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;

//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;
	AmplOutputMatrix<int, int, float>().dump(cout, rec.find<Matrix<int, int, float> >("schedule"));
//	std::cerr << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "]" << std::cerr;

	return EXIT_SUCCESS;
}

