/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <crown/frequency.h>

#include <pelib/AmplOutput.hpp>
#include <pelib/AmplInput.hpp>

using namespace std;
using namespace pelib;

int
main(int argc, char **argv)
{
	// Open and read input file
	ifstream taskgraph(argv[1], std::ios::in);

	Algebra input = AmplInput(AmplInput::floatHandlers()).parse(taskgraph);
	taskgraph.close();

	cout << setprecision(6)
	<< setiosflags(ios::fixed)
	<< setiosflags(ios::showpoint);
	
	input = frequency_height(input);

	// Garbage to make output to look like Ampl/Gurobi output
	cout << "Presolve eliminates 0 constraints and 0 variables." << endl;
	cout << "Substitution eliminates 0 variables." << endl;
	cout << "Adjusted problem:" << endl;
	cout << "0 variables:" << endl;
	cout << "	0 binary variables" << endl;
	cout << "	0 linear variable" << endl;
	cout << "0 constraints, all linear; 0 nonzeros" << endl;
	cout << "0 linear objective; 0 nonzero." << endl;
	cout << endl;
	cout << ": timelim 300" << endl;

	float m = input.find<Scalar<float> >("m")->getValue();
	float M = input.find<Scalar<float> >("M")->getValue();
	if(m < 0 && M > 0)
	{
		cout << "Height heuristic: infeasible" << endl;
		cout << "29 simplex iterations" << endl;
		cout << "plus 1 simplex iteration for intbasis" << endl;
	}
	else
	{
		cout << "plus 1 simplex iteration for intbasis" << endl;
		cout << "Height heuristic: heuristic solution; nonoptimal 0" << endl;
		cout << "29 simplex iterations" << endl;
	}
	
	AmplOutput(AmplOutput::floatHandlers()).dump(cout, input);
	AmplOutput(AmplOutput::floatHandlers()).dump(cout, Scalar<float>("complexity", frequency_height_complexity(input)));

	return EXIT_SUCCESS;
}

